@@ -10,41 +10,59 @@ from sqlalchemy.exc import IntegrityError
 from sqlalchemy.orm import sessionmaker, scoped_session
 
 
-
-
 def save_newest_discussion_subj(session, e):
-    max_count = 10
+    """save a given discussion subject as newest discussion subject in the postgreSQL
+    Arguments:
+        session: the session to use
+        e: the event containing the discussion subject
+    """
+    entities = []
+    if 'context_annotations' in e['subj']['data']:
+        context_entity = e['subj']['data']['context_annotations']
+        for entity_data in context_entity:
+            entities.append(entity_data['entity']['name'])
 
     obj = DiscussionNewestSubj(
         type='twitter',
-        publication_doi=e.data['obj']['data']['doi'],
-        sub_id=e.data['subj']['pid'],
+        publication_doi=e['obj']['data']['doi'],
+        sub_id=e['subj_id'],
         created_at=datetime.datetime.now(),
-        score=e.data['subj']['processed']['score'],
-        bot_rating=e.data['subj']['processed']['bot_rating'],
-        followers=e.data['subj']['processed']['followers'],
-        sentiment_raw=e.data['subj']['processed']['sentiment_raw'],
-        contains_abstract_raw=e.data['subj']['processed']['contains_abstract_raw'],
-        lang=e.data['subj']['data']['lang'],
-        location=e.data['subj']['processed']['location'],
-        source=e.data['subj']['data']['source'],
-        subj_type=e.data['subj']['processed']['tweet_type'],
-        question_mark_count=e.data['subj']['processed']['question_mark_count'],
-        exclamation_mark_count=e.data['subj']['processed']['exclamation_mark_count'],
-        length=e.data['subj']['processed']['length'],
-        entities=json.dumps(e['subj']['data']['context_annotations'])
+        score=e['subj']['processed']['score'],
+        bot_rating=e['subj']['processed']['bot_rating'],
+        followers=e['subj']['processed']['followers'],
+        sentiment_raw=e['subj']['processed']['sentiment_raw'],
+        contains_abstract_raw=e['subj']['processed']['contains_abstract_raw'],
+        lang=e['subj']['data']['lang'],
+        location=e['subj']['processed']['location'],
+        source=e['subj']['data']['source'],
+        subj_type=e['subj']['processed']['tweet_type'],
+        question_mark_count=e['subj']['processed']['question_mark_count'],
+        exclamation_mark_count=e['subj']['processed']['exclamation_mark_count'],
+        length=e['subj']['processed']['length'],
+        entities=json.dumps(entities)
     )
 
     table = DiscussionNewestSubj
-    key = {'publication_doi': e.data['obj']['data']['doi'], 'type': 'twitter'}
-    count = session.query(table).filter_by(**key).count()
-    print(count)
+    key = {'publication_doi': e['obj']['data']['doi'], 'type': 'twitter'}
+    oldest = session.query(table).filter_by(**key).order_by(DiscussionNewestSubj.created_at).first()
+
+    if oldest:
+        oldest.sub_id = obj.sub_id
+        oldest.created_at = obj.created_at
+        oldest.score = obj.score
+        oldest.bot_rating = obj.bot_rating
+        oldest.followers = obj.followers
+        oldest.sentiment_raw = obj.sentiment_raw
+        oldest.contains_abstract_raw = obj.contains_abstract_raw
+        oldest.lang = obj.lang
+        oldest.location = obj.location
+        oldest.source = obj.source
+        oldest.subj_type = obj.subj_type
+        oldest.question_mark_count = obj.question_mark_count
+        oldest.exclamation_mark_count = obj.exclamation_mark_count
+        oldest.length = obj.length
+        oldest.entities = obj.entities
 
-    if count >= max_count:
-        # update oldest
-        oldest = session.query(table).filter_by(**key).order_by(DiscussionNewestSubj.created_at).first()
-        print(oldest)
-        oldest = obj
         session.commit()
         return True
 
@@ -54,7 +72,6 @@ def save_newest_discussion_subj(session, e):
 
 
 class DAO(object):
-    # session = None
 
     def __init__(self):
         host_server = os.environ.get('POSTGRES_HOST', 'postgres')
@@ -63,22 +80,19 @@ class DAO(object):
         db_username = urllib.parse.quote_plus(str(os.environ.get('POSTGRES_USER', 'streams')))
         db_password = urllib.parse.quote_plus(str(os.environ.get('POSTGRES_PASSWORD', 'REPLACE_ME')))
 
-        # ssl_mode = urllib.parse.quote_plus(str(os.environ.get('ssl_mode','prefer')))
         DATABASE_URL = 'postgresql://{}:{}@{}:{}/{}'.format(db_username, db_password, host_server,
                                                             db_server_port, database_name)
-        print(DATABASE_URL)
-        # engine = create_engine('postgresql+psycopg2://streams:REPLACE_ME@postgres:5432/amba')
+        logging.debug(DATABASE_URL)
         self.engine = create_engine(DATABASE_URL, pool_size=20, max_overflow=0)
         Base.metadata.create_all(self.engine)
-        # database = databases.Database(DATABASE_URL)
-
-        # Session = sessionmaker(bind=engine)
-        # session_factory = sessionmaker(bind=self.engine)
-        # Session = scoped_session(session_factory)
-        # self.session = Session()
 
     @staticmethod
     def save_object(session, obj):
+        """save a given sqalchemy object
+        Arguments:
+            session: the session to use
+            obj: the object to store
+        """
         try:
             session.add(obj)
             session.commit()
@@ -88,11 +102,21 @@ class DAO(object):
 
     @staticmethod
     def object_as_dict(obj):
+        """transform a sqalchemy object to a dict
+        Arguments:
+            obj: the object to transform
+        """
         return {c.key: getattr(obj, c.key)
                 for c in inspect(obj).mapper.column_attrs}
 
     @staticmethod
     def get_object(session, table, key):
+        """get the (first) object in a given table using a given session for a given key
+        Arguments:
+            session: the session to use
+            table: the table to be used
+            key: what to use to filter and find
+        """
         result = session.query(table).filter_by(**key).first()
         if not result:
             return None
@@ -100,6 +124,13 @@ class DAO(object):
 
     @staticmethod
     def save_or_update_count(session, obj, table, kwargs):
+        """save or update the count of an table with count
+        Arguments:
+            session: the session to use
+            obj: the object to use
+            table: the table to be used
+            kwargs: what to use to filter and find
+        """
         obj_db = DAO.get_object(session, table, kwargs)
         if obj_db:
             # add count to existing object
@@ -112,6 +143,13 @@ class DAO(object):
 
     @staticmethod
     def save_if_not_exist(session, obj, table, kwargs):
+        """save and object only if not already exiting
+        Arguments:
+            session: the session to use
+            obj: the object to use
+            table: the table to be used
+            kwargs: what to use to filter and find
+        """
         obj_db = DAO.get_object(session, table, kwargs)
         if obj_db:
             return obj_db
@@ -120,14 +158,16 @@ class DAO(object):
         return obj
 
     def get_publication(self, doi):
+        """get a publication using a doi from the postgreSQL, if available add authors, fields of study and sources
+        Arguments:
+            doi: the doi to use
+        """
         session_factory = sessionmaker(bind=self.engine)
         Session = scoped_session(session_factory)
 
-        # todo add sources
         params = {'doi': doi, }
 
         session = Session()
-        # pub = session.query(Publication).filter_by(doi=doi).all()
         p = text("""SELECT * FROM publication WHERE doi=:doi""")
         p = p.bindparams(bindparam('doi'))
         resultproxy = session.execute(p, params)
@@ -165,6 +205,10 @@ class DAO(object):
         return result
 
     def save_publication(self, publication_data):
+        """save a publication using given publication data
+        Arguments:
+            publication_data: the publication data to be saved
+        """
         session_factory = sessionmaker(bind=self.engine)
         Session = scoped_session(session_factory)
         session = Session()
@@ -175,12 +219,12 @@ class DAO(object):
                                   citation_count=publication_data['citation_count'],
                                   title=publication_data['title'],
                                   normalized_title=publication_data['normalized_title'],
-                                  abstract=publication_data['abstract'])
+                                  abstract=publication_data['abstract'],
+                                  license=publication_data['license'])
         publication = self.save_if_not_exist(session, publication, Publication, {'doi': publication.doi})
 
         logging.debug('publication.doi')
         logging.debug(publication.doi)
-        # logging.warning(publication.id)
 
         authors = publication_data['authors']
         for author_data in authors:
@@ -195,7 +239,7 @@ class DAO(object):
         if 'source_id' in publication_data:
             sources = publication_data['source_id']
             for sources_data in sources:
-                source = Source(title=sources_data['title'], url=sources_data['url'])  # todo no doi url ?
+                source = Source(title=sources_data['title'], url=sources_data['url'])
                 source = self.save_if_not_exist(session, source, Source, {'title': source.title})
 
                 if source.id:
@@ -218,15 +262,11 @@ class DAO(object):
 
         session.close()
         return publication
-        # todo add perculator!!!!!!
-        # use different names for config until we remove gql?
-        # publicationCitations = PublicationCitations()
-        # publicationReferences = PublicationReferences(**author_data)
 
     def save_discussion_data(self, event_data):
         """save a discussion data row from event data
 
-        Argumetns:
+        Arguments:
             event_data: to be saved
         """
         session_factory = sessionmaker(bind=self.engine)
@@ -245,7 +285,8 @@ class DAO(object):
                     publication_entity = DiscussionDataPoint(
                         **{'publication_doi': publication_doi, 'discussion_data_point_id': entity.id, 'count': 1})
                     self.save_or_update_count(session, publication_entity, DiscussionDataPoint,
-                                              {'publication_doi': publication_doi, 'discussion_data_point_id': entity.id})
+                                              {'publication_doi': publication_doi,
+                                               'discussion_data_point_id': entity.id})
 
         if 'words' in event_data['subj']['processed']:
             words = event_data['subj']['processed']['words']
@@ -255,7 +296,8 @@ class DAO(object):
 
                 if word.id:
                     publication_words = DiscussionDataPoint(
-                        **{'publication_doi': publication_doi, 'discussion_data_point_id': word.id, 'count': words_data[1]})
+                        **{'publication_doi': publication_doi, 'discussion_data_point_id': word.id,
+                           'count': words_data[1]})
                     self.save_or_update_count(session, publication_words, DiscussionDataPoint,
                                               {'publication_doi': publication_doi, 'discussion_data_point_id': word.id})
 
@@ -270,12 +312,13 @@ class DAO(object):
                     publication_h = DiscussionDataPoint(
                         **{'publication_doi': publication_doi, 'discussion_data_point_id': hashtag.id, 'count': 1})
                     self.save_or_update_count(session, publication_h, DiscussionDataPoint,
-                                              {'publication_doi': publication_doi, 'discussion_data_point_id': hashtag.id})
+                                              {'publication_doi': publication_doi,
+                                               'discussion_data_point_id': hashtag.id})
 
         if 'name' in event_data['subj']['processed']:
             author = DiscussionData(value=event_data['subj']['processed']['name'], type='name')
             author = self.save_if_not_exist(session, author, DiscussionData,
-                                            {'value': author.value, 'type': 'author'})
+                                            {'value': author.value, 'type': 'name'})
 
             if author.id:
                 publication_author = DiscussionDataPoint(**{'discussion_data_point_id': author.id, 'count': 1,
@@ -295,14 +338,15 @@ class DAO(object):
                                           {'publication_doi': publication_doi, 'discussion_data_point_id': location.id})
 
         if 'tweet_type' in event_data['subj']['processed']:
-            type = DiscussionData(value=event_data['subj']['processed']['tweet_type'], type='tweet_type')
-            type = self.save_if_not_exist(session, type, DiscussionData, {'value': type.value, 'type': 'type'})
+            dd_type = DiscussionData(value=event_data['subj']['processed']['tweet_type'], type='tweet_type')
+            dd_type = self.save_if_not_exist(session, dd_type, DiscussionData,
+                                             {'value': dd_type.value, 'type': 'tweet_type'})
 
-            if type.id:
-                publication_type = DiscussionDataPoint(**{'discussion_data_point_id': type.id, 'count': 1,
+            if dd_type.id:
+                publication_type = DiscussionDataPoint(**{'discussion_data_point_id': dd_type.id, 'count': 1,
                                                           'publication_doi': publication_doi})
                 self.save_or_update_count(session, publication_type, DiscussionDataPoint,
-                                          {'publication_doi': publication_doi, 'discussion_data_point_id': type.id})
+                                          {'publication_doi': publication_doi, 'discussion_data_point_id': dd_type.id})
 
         if 'lang' in event_data['subj']['data']:
             lang = DiscussionData(value=event_data['subj']['data']['lang'], type='lang')
@@ -327,3 +371,33 @@ class DAO(object):
 
         session.close()
         return True
+
+    def save_publication_not_found(self, doi, pub_is_done):
+        """store (or update) a not found publication doi as well as why
+        Arguments:
+            doi: the doi that could not be found
+            pub_is_done: what is missing from the publication
+        """
+        session_factory = sessionmaker(bind=self.engine)
+        Session = scoped_session(session_factory)
+        session = Session()
+
+        obj = PublicationNotFound(
+            publication_doi=doi,
+            last_try=datetime.datetime.now(),
+            pub_missing=pub_is_done,
+        )
+
+        table = PublicationNotFound
+        kwargs = {'publication_doi': doi}
+        obj_db = DAO.get_object(session, table, kwargs)
+        if obj_db:
+            # add count to existing object
+            obj_db.last_try = datetime.datetime.now()
+            session.commit()
+            session.close()
+            return obj_db
+
+        DAO.save_object(session, obj)
+        session.close()
+        return obj
