@@ -3,47 +3,36 @@ import logging
 import os
 import threading
 import time
-
 from .event_stream_base import EventStreamBase
-
-from kafka import KafkaConsumer, KafkaProducer
+from kafka import KafkaConsumer
 from kafka.vendor import six
-
-from multiprocessing import Process, Queue, current_process, freeze_support, Pool, Value
+from multiprocessing import Queue, Pool, Value
 
 
-# idee
-# import eventstream reader
-# class inherince
-# override for each message method, use var as string?
-#     -> goal of eventstream
-#
-# o1 here is function, do everything else (mulitple threads etc)
-#
-# o2 here is a class you can ran as you wish
-#
-# o3 (1+2) eventstream class has functions to do multithreads with the class
-#
-# consumer
-# producer
-# consumer producer
-#
-# -> event stream problem (handle multiple or just one each?)
-# eventstreap processor process producer1, consumer2,
-
-# todo util
-def throughput_statistics(v, time_delta):
+def throughput_statistics(v, time_delta, no_throughput_counter=0):
     """show and setup in own thread repeatedly how many events are processed
-
+        restarts if counter of no throughput is 10 (10 timed deltas with no data processed)
     Arguments:
         v: the value
         time_delta: time delta we wan't to monitor
+        no_throughput_counter: counter of no throughput
     """
     logging.warning("THROUGHPUT: %d / %d" % (v.value, time_delta))
+
+    if v.value == 0:
+        no_throughput_counter += 1
+    else:
+        no_throughput_counter = 0
+    if no_throughput_counter == 10:
+        logging.warning('Exit Container because of no data throughput')
+        os.system("pkill -9 python")  # allows killing of multiprocessing programs
+
     with v.get_lock():
         v.value = 0
 
-    threading.Timer(time_delta, throughput_statistics, args=[v, time_delta]).start()
+    api_limit_thread = threading.Timer(time_delta, throughput_statistics, args=[v, time_delta, no_throughput_counter])
+    api_limit_thread.daemon = True
+    api_limit_thread.start()
 
 
 class EventStreamConsumer(EventStreamBase):
@@ -55,6 +44,7 @@ class EventStreamConsumer(EventStreamBase):
     state = "unlinked"
     topics = False
     consumer = False
+    throughput_statistics_running = False
 
     task_queue = Queue()
     process_number = 4
@@ -88,9 +78,7 @@ class EventStreamConsumer(EventStreamBase):
                 for relation_type in self.relation_type:
                     self.topics.append(self.get_topic_name(state=state, relation_type=relation_type))
 
-        # self.topic_name = 'tweets'
         logging.debug(self.log + "get consumer for topic: %s" % self.topics)
-        # consumer.topics()
         self.consumer = KafkaConsumer(group_id=self.group_id,
                                       bootstrap_servers=self.bootstrap_servers, api_version=self.api_version,
                                       consumer_timeout_ms=self.consumer_timeout_ms)
@@ -111,14 +99,13 @@ class EventStreamConsumer(EventStreamBase):
         if not self.consumer:
             self.create_consumer()
 
-        if not self.counter:
+        if self.throughput_statistics_running and not self.counter:
             self.counter = Value('i', 0)
             counter_time = 10
-            threading.Timer(counter_time, throughput_statistics, args=[self.counter, counter_time]).start()
+            api_limit_thread = threading.Timer(counter_time, throughput_statistics, args=[self.counter, counter_time])
+            api_limit_thread.daemon = True
+            api_limit_thread.start()
 
-        # Start worker processes
-        # for i in range(self.process_number):
-        #     Process(target=self.on_message, args=(self.task_queue, )).start()
         pool = Pool(self.process_number, self.worker, (self.task_queue,))
 
         while self.running:
@@ -128,7 +115,6 @@ class EventStreamConsumer(EventStreamBase):
                     if self.counter:
                         with self.counter.get_lock():
                             self.counter.value += 1
-                    # logging.warning('msg in consumer %s' % msg.value)
                     self.task_queue.put(json.loads(msg.value.decode('utf-8')))
 
             except Exception as exc:
